/*
 *  The "this" reference
 *   
 *  Use Chrome Developer Tools to run and debug
 *  Add break point and reload page to inspect!
 */
'use strict';

// In global code this is the window (global) object ----------------------
this.a = 10;
console.log(window.a === this.a);


// Function in global scoope   ------------------
function globalIsOwning() {
    return this;       // If strict mode undefined, in non-strict mode
    // set to global (window)
}
console.log(globalIsOwning() === window);

// Constructor function -------------------------
var Person = function (name, age) {
    this.name = name; // "this" is the actual object
    this.age = age;

};
var p1 = new Person("Otto", 13);

console.log(p1.constructor.name);  // Follow __proto__ chain ...
console.log(Object.getPrototypeOf(p1).constructor.name); // ... same as

// Methods (owning object) -----------------------------------------
var fooIsOwning = {
    bar: function () {
        return this;
    }
};

console.log(fooIsOwning.bar() === fooIsOwning);

var test = fooIsOwning.bar;
console.log(test() === fooIsOwning); // Strict mode undef. else global is owning

// More on owning object
var objA = {x: 10};
var objB = {
    x: 20,
    test: function () {
        console.log(this === objB);
        console.log(this.x);
    }
};

// this is determined as objB object 
objB.test(); // true, 20

objA.test = objB.test;

// Now this is objA object!! 
objA.test(); // false, 10


// Functions ----------------------------------------

function foo() {
    function bar() {    // No owning object (owned by invisible acivation object)
        console.log(this); // Strict mode undef. else global
    }
    bar();
}
foo();

function foo2() {
    var bar2 = (function () {
        console.log(this);   // Strict mode undef. else global   
    });
    bar2();
}
foo2();


// Preserving this, store as "self" -----------------------

var myObject = {
    value: 0,
    increment: function (inc) {
        // myObject owning
        this.value += (typeof inc === 'number') ? inc : 1;
    },
    double: function () {
        var self = this;   // HERE!
        function helper() {
            console.log("This is" + this);  // Undef
            console.log("Self is " + self);
            self.value *= 2;
        }
        helper();
    }
};

myObject.increment(3);
console.log(myObject.double());
console.log(myObject.value);

// Specifying this using bind()----------------
this.x = 9;      // this is global (window)
var module = {
    x: 81,
    getX: function () {
        return this.x;
    }
};

console.log(module.getX()); // 81

var getX = module.getX;
//console.log(getX()); // // Strict mode undef. (exception) else global (result = 9)

// Create a new function with 'this' bound to module
var boundGetX = getX.bind(module);
console.log(boundGetX()); // 81
