package jdbc.db;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

/**
 * Low level database access
 * 
 * Singleton because this should live during application execution
 * (other objects are created for page, request, session ,etc..)
 */
public enum Database {
    INSTANCE;

    private final PersonDAO dao = new PersonDAO(getDataSource());

    public PersonDAO get() {
        return dao;
    }

    private DataSource ds;

    // The preffered way to access the database, See web.xml and context.xml
    private DataSource getDataSource() {
        if (ds == null) {
            // Will create pooled connections, good!
            Context c;
            try {
                c = new InitialContext();
                ds = (DataSource) c.lookup("java:comp/env/jdbc/persons");
            } catch (NamingException ex) {
                Logger.getAnonymousLogger().log(Level.SEVERE, null, ex);
            }
        }
        return ds;
    }

}
