package jdbc.db;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.sql.DataSource;

/**
 * Implementation of Data Access Object (NON ORM, String based)
 *
 * NOTE: This uses JavaDb Derby (possible SQL syntax problems?)
 *
 * @author hajo
 */
public class PersonDAO implements IDAO {

    private final static Logger LOG = Logger.getAnonymousLogger();
    private final DataSource ds;

    public PersonDAO(DataSource ds) {
        this.ds = ds;
    }

    @Override
    public List<String> getPersons() throws SQLException {
        Logger.getAnonymousLogger().log(Level.INFO, "getPersons");
        String sql = "select * from Persons";  // Raw SQL
        List<String> result; 
        // NOTE: Connection is in auto commit mode, no explicit transaction
        // demarcation needed (like tx.begin() tx.end())
        try (Connection c = ds.getConnection();
                Statement stmt = c.createStatement();
                ResultSet rs = stmt.executeQuery(sql);) {
            result = buildResult(rs);
        }
        return result;
    }

    @Override
    public String getById(Long id) throws SQLException {
        String sql = "select * from Persons where id=" + id;
        List<String> result;
        try (Connection c = ds.getConnection();
                Statement stmt = c.createStatement();
                ResultSet rs = stmt.executeQuery(sql);) {
            result = buildResult(rs);
        }
        return result.get(0);
    }

    @Override
    public void add(String person) throws SQLException {
        // Uuuugglllyyyy!
        String sql = "insert into Persons (name) values ('" + person + "')";
        LOG.log(Level.INFO, sql);
        try (Connection c = ds.getConnection();
                Statement stmt = c.createStatement();) {
            stmt.executeUpdate(sql);
        }
    }

    @Override
    public void delete(Long id) throws SQLException {
        String sql = "delete from Persons where id=" + id;
        try (Connection c = ds.getConnection();
                Statement stmt = c.createStatement();) {
            stmt.executeUpdate(sql);
        }
    }

    private List<String> buildResult(ResultSet rs) throws SQLException {
        List<String> result = new ArrayList();
        StringBuilder sb = new StringBuilder();
        while (rs.next()) {
            int id = rs.getInt("id");
            String name = rs.getString("name");
            sb.append(id).append(";")
                    .append(name).append(";");
            result.add(sb.toString());
            sb.setLength(0);
        }
        return result;
    }

}
