package edu.chl.hajo.shop.test;

import edu.chl.hajo.shop.core.Product;
import edu.chl.hajo.shop.dao.CustomerRegistry;
import edu.chl.hajo.shop.dao.OrderBook;
import edu.chl.hajo.shop.dao.ProductCatalogue;
import java.io.IOException;
import java.io.PrintWriter;
import javax.enterprise.inject.Default;
import javax.enterprise.inject.Produces;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;

/**
 *
 * @author hajo
 */
@WebServlet(name = "DbSetupServlet", urlPatterns = {"/setup"})
public class DbSetupServlet extends HttpServlet {

    // This will trigger creation of all tables
    @Inject
    ProductCatalogue pc;
    @Inject
    CustomerRegistry cr;
    @Inject
    OrderBook ob;
    @Inject
    UserTransaction utx;

    @Override
    protected void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        clearAll();

        //String productCode, String name, double price
        pc.create(new Product("NN1", "Tvål", 35.90));
        //cr.create(new Customer(email, firstName, lastName, address, cart));
        //ob.create(new PurchaseOrder(orderNumber, items, customer, PurchaseOrder.State.SHIPPED));

        response.setContentType("text/html;charset=UTF-8");
        try (PrintWriter out = response.getWriter()) {
            /* TODO output your page here. You may use following sample code. */
            out.println("<!DOCTYPE html>");
            out.println("<html>");
            out.println("<head>");
            out.println("<title>Test Servlet</title>");
            out.println("</head>");
            out.println("<body>");
            out.println("<h1>Done</h1>");
            out.println("<p>Inspect JavaDB, tables created and some data inserted (possibly refresh, tables)</p>");
            out.println("</body>");
            out.println("</html>");
        }

    }

    // Need a standalone em to remove testdata between tests
    // No em accessible from interfaces
    @PersistenceContext(unitName = "jpa_shop_pu")
    @Produces
    @Default
    EntityManager em;

    // Order matters
    private void clearAll() throws ServletException {
        try {
            utx.begin();
            em.joinTransaction();
            em.createQuery("delete from PurchaseOrder").executeUpdate();
            em.createQuery("delete from Customer").executeUpdate();
            em.createQuery("delete from Product").executeUpdate();
            utx.commit();
        } catch (NotSupportedException | SystemException | RollbackException | HeuristicMixedException | HeuristicRollbackException | SecurityException | IllegalStateException e) {
            throw new ServletException("Couldn't clear database");
        }
    }

}
