package jeecomp.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;
import javax.annotation.PostConstruct;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Named;

/**
 * Model object  
 *
 * This will later be handled by the database
 *
 * @author hajo
 */
@Named
@ApplicationScoped
// @Stateless Am EJB (DAO) later if in persistence layer
public class PersonRegistry {

    private final List<Person> data = new ArrayList<>();

    @PostConstruct
    public void post() {
        generateData();
    }

    public void add(Person p) {
        if (!data.contains(p)) {
             data.add(p);
        }   
    }

    public void update(Person p) {
        Person old = find(p.getId());
        int i = data.indexOf(old);
        if (i > -1) {
            data.set(i, p);
        }
    }

    public void delete(Long id) {
        Person p = find(id);
        if (p != null) {
            data.remove(p);
        }
    }

    public Person find(Long id) {
        for (Person p : data) {
            if (p.getId().equals(id)) {
                return p;
            }
        }
        return null;
    }

    public Collection<Person> findAll() {
        return data;
    }

    public Collection<Person> findRange(int first, int n) {
        if( first < 0 || data.size() < first){
           return new ArrayList<>();
        } else if (first + n < data.size()) {
            return data.subList(first, first + n);
        } else {
            return data.subList(first, data.size());
        }
    }

    public int size() {
        return data.size();
    }

    // Emulating a database
    private void generateData() {
        String s = "Abbe Abel Abraham Acke Ada Adalbert Adam Adela Adele Adelia"
                + " Adelina Berit Berna Bernhard Bernhardina Bernt Berta"
                + " Bertel Berthold Bertil Bertina Bessie Beth Betsy Bettina"
                + " Sabine Salli Teddy Tekla Teodor Teodora Uno Urban Valfrida"
                + " Valle Yvette Yvonne";
        String[] names = s.split(" ");
        long id = 1L;
        Random r = new Random();
        for (String name : names) {
            data.add(new Person(null, names[r.nextInt(names.length-1)]));
        }

    }

}
